//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
//THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.
//---------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Globalization;
using System.Collections;
using System.Reflection;


namespace DataBindingInvestigation
{
    public class Form1 : Form
    {
        private TextBox nameTextBox;
        private TextBox dateTextBox;
        private ErrorProvider errorProvider1;
        private IContainer components;
        private TextBox salaryTextBox;
    
        public Form1()
        {
            InitializeComponent();
            this.Load += new EventHandler(Form1_Load);
        }

        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Form1());
        }

        void Form1_Load(object sender, EventArgs e)
        {
            ShowAutomaticFormatting();
            //ShowUpdateMode();
            //ShowHandlingOfDBNull();
            //ShowErrorHandling();
        }

        private void ShowAutomaticFormatting()
        {
            DataTable table1 = new DataTable();
            table1.Columns.Add("Name", typeof(string));
            table1.Columns.Add("Hire Date", typeof(DateTime));
            table1.Columns.Add("Starting salary", typeof(int));

            table1.Rows.Add(new object[] { "Claus Hansen", new DateTime(2003, 5, 5), 
                63000 });
            Binding dateBinding = new Binding("Text", table1, "Hire Date");

            // Set FormattingEnabled and FormatString directly on the Binding object.
            // The "D" format specifier indicates the long date pattern.
            dateBinding.FormattingEnabled = true;
            dateBinding.FormatString = "D";

            // Add the binding to the dateTextBox binding collection.
            dateTextBox.DataBindings.Add(dateBinding);

            // Set FormattingEnabled as part of the 
            // ControlBindingsCollection.Add method.
            Binding salaryBinding = salaryTextBox.DataBindings.Add("text", table1,
                "Starting Salary", true);

            // The "c" format specifier indicates currency format.
            salaryBinding.FormatString = "c";

            Binding nameBinding = nameTextBox.DataBindings.Add("Text", table1,
                "Name", true);
        }

        private void ShowUpdateMode()
        {
            DataTable table1 = new DataTable();
            table1.Columns.Add("Name", typeof(string));
            table1.Columns.Add("Hire Date", typeof(DateTime));
            table1.Columns.Add("Starting salary", typeof(int));

            table1.Rows.Add(new object[] { "Claus Hansen", new DateTime(2003, 5, 5), 63000 });

            // Hire Date in table1 is updated when any change to the hire date
            // text box is made.
            Binding dateBinding2 = dateTextBox.DataBindings.Add("Text", table1, "Hire Date", true,
                DataSourceUpdateMode.OnPropertyChanged);
            dateBinding2.FormatString = "D";

            // Starting Salary in table1 is updated when the starting salary text box
            // loses focus and the value passes validation.
            Binding salaryBinding = salaryTextBox.DataBindings.Add("text", table1, "Starting salary", true,
                DataSourceUpdateMode.OnValidation);
            salaryBinding.FormatString = "c";

            Binding nameBinding = nameTextBox.DataBindings.Add("Text", table1, "Name", true);
        }

        private void ShowHandlingOfDBNull()
        {
            DataTable table1 = new DataTable();
            table1.Columns.Add("Name", typeof(string));
            table1.Columns.Add("Hire Date", typeof(DateTime));
            table1.Columns.Add("Starting salary", typeof(int));

            // Initialize Hire Date to DBNull.
            table1.Rows.Add(new object[] { "Claus Hansen", DBNull.Value, 63000 });

            // If Hire Date is null or DBNull, change to "New Hire".
            Binding dateBinding2 = dateTextBox.DataBindings.Add("Text", table1, "Hire Date", true,
                DataSourceUpdateMode.OnValidation, "New Hire", "D");

            Binding salaryBinding = salaryTextBox.DataBindings.Add("text", table1, "Starting salary", true);
            salaryBinding.FormatString = "c";

            Binding nameBinding = nameTextBox.DataBindings.Add("Text", table1, "Name", true);
        }

        private void ShowErrorHandling()
        {
            BindingSource employeeBindingSource = new BindingSource();
            employeeBindingSource.Add(new Employee("Hansen", "Claus", 63000, new DateTime(2002, 5, 5)));
            employeeBindingSource.Add(new Employee("Han", "Mu", 54000, new DateTime(2001, 3, 4)));
            nameTextBox.DataBindings.Add("Text", employeeBindingSource, "LastName", true);
            dateTextBox.DataBindings.Add("Text", employeeBindingSource, "StartDate",
              true, DataSourceUpdateMode.OnValidation);

            Binding salaryBinding = salaryTextBox.DataBindings.Add("text", employeeBindingSource, "Salary",
                true, DataSourceUpdateMode.OnValidation, "Unknown", "c");
            salaryBinding.BindingComplete += new BindingCompleteEventHandler(salaryBinding_BindingComplete);
        }


        void salaryBinding_BindingComplete(object sender, BindingCompleteEventArgs e)
        {
            // If the BindingComplete state is anything other than success, 
            // set the ErrorProvider to the error message.
            if (e.BindingCompleteState != BindingCompleteState.Success)
                errorProvider1.SetError((Control)e.Binding.BindableComponent, e.ErrorText);
            else
                errorProvider1.SetError((Control)e.Binding.BindableComponent, "");
        }

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            this.nameTextBox = new System.Windows.Forms.TextBox();
            this.dateTextBox = new System.Windows.Forms.TextBox();
            this.salaryTextBox = new System.Windows.Forms.TextBox();
            this.errorProvider1 = new System.Windows.Forms.ErrorProvider(this.components);
            ((System.ComponentModel.ISupportInitialize)(this.errorProvider1)).BeginInit();
            this.SuspendLayout();
            // 
            // nameTextBox
            // 
            this.nameTextBox.Location = new System.Drawing.Point(12, 32);
            this.nameTextBox.Name = "nameTextBox";
            this.nameTextBox.Size = new System.Drawing.Size(129, 20);
            this.nameTextBox.TabIndex = 0;
            // 
            // dateTextBox
            // 
            this.dateTextBox.Location = new System.Drawing.Point(12, 58);
            this.dateTextBox.Name = "dateTextBox";
            this.dateTextBox.Size = new System.Drawing.Size(129, 20);
            this.dateTextBox.TabIndex = 1;
            // 
            // salaryTextBox
            // 
            this.salaryTextBox.Location = new System.Drawing.Point(12, 84);
            this.salaryTextBox.Name = "salaryTextBox";
            this.salaryTextBox.Size = new System.Drawing.Size(129, 20);
            this.salaryTextBox.TabIndex = 2;
            // 
            // errorProvider1
            // 
            this.errorProvider1.ContainerControl = this;
            // 
            // Form1
            // 
            this.ClientSize = new System.Drawing.Size(292, 266);
            this.Controls.Add(this.salaryTextBox);
            this.Controls.Add(this.dateTextBox);
            this.Controls.Add(this.nameTextBox);
            this.Name = "Form1";
            ((System.ComponentModel.ISupportInitialize)(this.errorProvider1)).EndInit();
            this.ResumeLayout(false);
            this.PerformLayout();
        }
    }


    public class Employee
    {
        private string lastNameValue;
        private string firstNameValue;
        private int salaryValue;
        private DateTime startDateValue;

        public Employee(string lastName, string firstName, 
            int salary, DateTime startDate)
        {
            LastName = lastName;
            FirstName = firstName;
            Salary = salary;
            StartDate = startDate;
        }
        
        public string LastName
        {
            get { return lastNameValue; }
            set { lastNameValue = value; }
        }
        public string FirstName
        {
            get { return firstNameValue; }
            set { firstNameValue = value; }
        }

        public int Salary
        {
            get { return salaryValue; }
            set
            {
                if (value < 20000)
                    throw new Exception("Salary cannot be less than $20,000");
                else salaryValue = value;
            }
        }

        public DateTime StartDate
        {
            get { return startDateValue; }
            set
            {
                startDateValue = value;
            }
        }

        public override string ToString()
        {
            return LastName + ", " + FirstName + "\n" + 
                "Salary:" + salaryValue + "\n" +
                "Start date:" + startDateValue;
        }
    }
}