'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports System.IO
Imports System.Globalization
Imports System.Collections
Imports System.Reflection

Class Form1
    Inherits Form
    
    Private nameTextBox As TextBox
    Private dateTextBox As TextBox
    Private errorProvider1 As ErrorProvider
    Private salaryTextBox As TextBox
    
    Public Sub New()
        InitializeComponent()
    End Sub   
    
    <STAThread()>  _
    Shared Sub Main() 
        Application.EnableVisualStyles()
        Application.SetCompatibleTextRenderingDefault(False)
        Application.Run(New Form1())
    
    End Sub   
    
    Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) _
        Handles Me.Load

        ShowAutomaticFormatting()
        'ShowUpdateMode()
        'ShowHandlingOfDBNull()
        'ShowErrorHandling()
    End Sub

    Private Sub ShowAutomaticFormatting() 
        Dim table1 As New DataTable()
        table1.Columns.Add("Name", GetType(String))
        table1.Columns.Add("Hire Date", GetType(DateTime))
        table1.Columns.Add("Starting salary", GetType(Integer))
        
        table1.Rows.Add(New Object() {"Claus Hansen", New DateTime(2003, 5, 5), 63000})
        Dim dateBinding As New Binding("Text", table1, "Hire Date")
        
        ' Set FormattingEnabled and FormatString directly on the Binding object.
        ' The "D" format specifier indicates the long date pattern.
        dateBinding.FormattingEnabled = True
        dateBinding.FormatString = "D"
        
        ' Add the binding to the dateTextBox binding collection.
        dateTextBox.DataBindings.Add(dateBinding)
        
        ' Set FormattingEnabled as part of the 
        ' ControlBindingsCollection.Add method.
        Dim salaryBinding As Binding = salaryTextBox.DataBindings.Add("text", _
            table1, "Starting Salary", True)
        
        ' The "c" format specifier indicates currency format.
        salaryBinding.FormatString = "c"
        
        Dim nameBinding As Binding = nameTextBox.DataBindings.Add("Text", table1, _
            "Name", True)
    
    End Sub

    Private Sub ShowUpdateMode() 
        Dim table1 As New DataTable()
        table1.Columns.Add("Name", GetType(String))
        table1.Columns.Add("Hire Date", GetType(DateTime))
        table1.Columns.Add("Starting salary", GetType(Integer))
        
        table1.Rows.Add(New Object() {"Claus Hansen", New DateTime(2003, 5, 5), 63000})
        
        ' Hire Date in table1 is updated when any change to the hire date
        ' text box is made.
        Dim dateBinding2 As Binding = dateTextBox.DataBindings.Add("Text", table1, _
            "Hire Date", True, DataSourceUpdateMode.OnPropertyChanged)
        dateBinding2.FormatString = "D"
        
        ' Starting Salary in table1 is updated when the starting salary text box
        ' loses focus and the value passes validation.
        Dim salaryBinding As Binding = salaryTextBox.DataBindings.Add("text", table1, _
            "Starting salary", True, DataSourceUpdateMode.OnValidation)
        salaryBinding.FormatString = "c"
        
        Dim nameBinding As Binding = nameTextBox.DataBindings.Add("Text", table1, _
            "Name", True)
    
    End Sub
    
    Private Sub ShowHandlingOfDBNull() 
        Dim table1 As New DataTable()
        table1.Columns.Add("Name", GetType(String))
        table1.Columns.Add("Hire Date", GetType(DateTime))
        table1.Columns.Add("Starting salary", GetType(Integer))
        
        ' Initialize Hire Date to DBNull.
        table1.Rows.Add(New Object() {"Claus Hansen", DBNull.Value, 63000})
        
        ' If Hire Date is null or DBNull, change to "New Hire".
        Dim dateBinding2 As Binding = dateTextBox.DataBindings.Add("Text", table1, _
            "Hire Date", True, DataSourceUpdateMode.OnValidation, "New Hire", "D")
        
        Dim salaryBinding As Binding = salaryTextBox.DataBindings.Add("text", table1, _
            "Starting salary", True)
        salaryBinding.FormatString = "c"
        
        Dim nameBinding As Binding = nameTextBox.DataBindings.Add("Text", table1, _
            "Name", True)    
    End Sub
    
    Dim WithEvents salaryBinding As Binding

    Private Sub ShowErrorHandling() 
        Dim employeeBindingSource As New BindingSource()
        employeeBindingSource.Add(New Employee("Hansen", "Claus", 63000, New DateTime(2002, 5, 5)))
        employeeBindingSource.Add(New Employee("Han", "Mu", 54000, New DateTime(2001, 3, 4)))
        nameTextBox.DataBindings.Add("Text", employeeBindingSource, "LastName", True)
        dateTextBox.DataBindings.Add("Text", employeeBindingSource, "StartDate", _
            True, DataSourceUpdateMode.OnValidation)
        
        salaryBinding = salaryTextBox.DataBindings.Add("text", _
            employeeBindingSource, "Salary", True, DataSourceUpdateMode.OnValidation, "Unknown", "c")
       
    End Sub
     
    Sub salaryBinding_BindingComplete(ByVal sender As Object, _
        ByVal e As BindingCompleteEventArgs) Handles salaryBinding.BindingComplete

        ' If the BindingComplete state is anything other than success, 
        ' set the ErrorProvider to the error message.
        If e.BindingCompleteState <> BindingCompleteState.Success Then
            errorProvider1.SetError(CType(e.Binding.BindableComponent, Control), e.ErrorText)
        Else
            errorProvider1.SetError(CType(e.Binding.BindableComponent, Control), "")
        End If

    End Sub
     
    Private Sub InitializeComponent() 
        Me.nameTextBox = New System.Windows.Forms.TextBox()
        Me.dateTextBox = New System.Windows.Forms.TextBox()
        Me.salaryTextBox = New System.Windows.Forms.TextBox()
        Me.errorProvider1 = New ErrorProvider()
        Me.SuspendLayout()
        ' 
        ' nameTextBox
        ' 
        Me.nameTextBox.Location = New System.Drawing.Point(12, 32)
        Me.nameTextBox.Name = "nameTextBox"
        Me.nameTextBox.Size = New System.Drawing.Size(129, 20)
        Me.nameTextBox.TabIndex = 0
        ' 
        ' dateTextBox
        ' 
        Me.dateTextBox.Location = New System.Drawing.Point(12, 58)
        Me.dateTextBox.Name = "dateTextBox"
        Me.dateTextBox.Size = New System.Drawing.Size(129, 20)
        Me.dateTextBox.TabIndex = 1
        ' 
        ' salaryTextBox
        ' 
        Me.salaryTextBox.Location = New System.Drawing.Point(12, 84)
        Me.salaryTextBox.Name = "salaryTextBox"
        Me.salaryTextBox.Size = New System.Drawing.Size(129, 20)
        Me.salaryTextBox.TabIndex = 2
        ' 
        ' errorProvider1
        ' 
        Me.errorProvider1.ContainerControl = Me
        ' 
        ' Form1
        ' 
        Me.ClientSize = New System.Drawing.Size(292, 266)
        Me.Controls.Add(salaryTextBox)
        Me.Controls.Add(dateTextBox)
        Me.Controls.Add(nameTextBox)
        Me.Name = "Form1"
        Me.ResumeLayout(False)
        Me.PerformLayout()
    
    End Sub
End Class


Public Class Employee
    Private lastNameValue As String
    Private firstNameValue As String
    Private salaryValue As Integer
    Private startDateValue As DateTime
    
    Public Sub New(ByVal lastName As String, ByVal firstName As String, _
        ByVal salary As Integer, ByVal startDate As DateTime)
        lastNameValue = lastName
        firstNameValue = firstName
        salaryValue = salary
        startDateValue = startDate
    End Sub

    Public Property LastName() As String
        Get
            Return lastNameValue
        End Get
        Set(ByVal value As String)
            lastNameValue = value
        End Set
    End Property
    
    Public Property FirstName() As String 
        Get
            Return firstNameValue
        End Get
        Set
            firstNameValue = value
        End Set
    End Property 
    
    Public Property Salary() As Integer 
        Get
            Return salaryValue
        End Get
        Set
            If value < 20000 Then
                Throw New Exception("Salary cannot be less than $20,000")
            Else
                salaryValue = value
            End If
        End Set
    End Property 
    
    Public Property StartDate() As DateTime 
        Get
            Return startDateValue
        End Get
        Set
            startDateValue = value
        End Set
    End Property   
    
    Public Overrides Function ToString() As String 
        Return LastName & ", " & FirstName & vbLf & "Salary:" _
            & salaryValue & vbLf & "Start date:" & startDateValue
    End Function
End Class 

